#ifndef __DEBUG_H__
#define __DEBUG_H__
/*! \mainpage
 *  \author Janusz Dziedzic (l4mer)
 *  \date 31-08-2007
 *
 *  1. Please add debug.c and ntstatstr.c to your sources file \n
 *  2. Include debug.h \n
 *  2. Define DRIVER_DEBUG and DRIVER_NAME \n
 *  3. Use this module \n
 *
 *  \file debug.h
 *  \brief Driver Debug file.
 *  
 */

//
// DebugMask: 0xdmmmllll
// d - device: FDO | PDO
// m - module
// l - debug level
//

// Modules
#define DBGMSKMAIN    0x00010000
#define DBGMSKPNP	    0x00020000
#define DBGMSKPOWER   0x00040000
#define DBGMSKIOCTL   0x00080000
#define DBGMSKWRITE   0x00100000
#define DBGMSKREAD    0x00200000

// PDO | FDO
#define DBGMSKDEV     0xf0000000
#define DBGMSKFDO     0x10000000
#define DBGMSKPDO     0x20000000

// Debug Level
#define DBGLVLERROR	  0x00000001
#define DBGLVLWARN	  0x00000003
#define DBGLVLINFO    0x00000007
#define DBGLVLTRACE	  0x0000000F

#define DBGALLMODULES (DBGMSKMAIN | DBGMSKPNP | \
                       DBGMSKPOWER | DBGMSKIOCTL | \
                       DBGMSKWRITE | DBGMSKREAD)

/*! \fn VOID DriverDbgDumpData(PCHAR Buffer, ULONG BufferSize) 
    \brief Send data dump to kernel debuger.
    
    This function sends data dump to kernel debuger or DbgView programm.
    Dump data as hexadecimal and ASCII values.
    Use this function only for buffers that you allocated.
 
    \param Buffer is a pointer to user buffer
    \param BufferSize is a size of user buffer in Bytes
    \return VOID
    \see 
*/
VOID
DriverDbgDumpData(PCHAR Buffer, ULONG BufferSize);


/*! \fn VOID DriverDbgTest(VOID)
    \brief Internal function that tests DriverDbg module.
    
    This function is useful to tests DriverDbg module.
    Check correct behavior of DriverDbg implementation.
 
    \return VOID
*/
VOID
DriverDbgTest(VOID);


/*! \fn VOID DriverDbgSetDebugMask(ULONG NewDebugMask)
    \brief Set Debug Mask.
    
    This function sets debug mask.
 
    \param NewDebugMask new DebugMask value
    \return VOID
    \see DebugMask
*/
VOID
DriverDbgSetDebugMask(ULONG NewDebugMask);


#ifdef DRIVER_DEBUG

#if !defined(DRIVER_NAME)
#error !!! Please define DRIVER_NAME (eg. #define DRIVER_NAME "mydriver"). !!!
#endif

#define IRP_NAMES_MAX_LEN 32
#define PNP_MINOR_FUNC_NAMES_MAX_LEN 64

/*! \var ULONG DebugMask
    \brief Debug mask.
    
    DebugMask: 0xdmmmllll
      \n  d - device: FDO | PDO
      \n  m - module
      \n  l - debug level
*/
extern ULONG DebugMask; 

// Major, Minor function
extern const CHAR IrpNames[][IRP_NAMES_MAX_LEN];
extern const CHAR PnpMinorFuncNames[][PNP_MINOR_FUNC_NAMES_MAX_LEN];

// OSR status
extern PUCHAR OsrNTStatusToString(NTSTATUS Status);


/*! \def DriverDbgPrint(_level_, _module_, _format_)
    \brief Print message to Debuger.
    
    This fuction check level, module with DebugMask value,
    and next send message to Debuger.
  
   \param _level_ debug level, can be DBGLVLERROR, DBGLVLWARN, DBGLVLINFO, DBGLVLTRACE
                  This value is compared with DebugMask
   \param _module_ debug module, can be DBGMSKMAIN, DBGMSKPNP, DBGMSKPOWER ...
   \param _format_ format of message that we want send to Debuger eg.
                   ("%d some problems", DebugMask)
   \return VOID
   \see DebugMask
*/
#define DriverDbgPrint(_level_, _module_, _format_) \
  do { \
      if( ( (_level_) <= (DebugMask & 0xFFFF) )  && ( (_module_) & DebugMask ) ) \
      { \
          DbgPrint _format_; \
      } \
  } while(0)


/*! \def DriverDbgEnterFunc(_module_)
 *  \brief Print name of function.
 * 
 *  You can use this macro in begining of all function. With a 
 *  correct DebugMask you will have trace of all called fuctions.
 *  Useful with DriverDbgLeaveFunc and DriverDbgLeaveFuncWithStatus.
 *
 * 
 *  \param _module_ debug module
 *  \return VOID
 *  \see DebugMask, DriverDbgPrint, DriverDbgSetDebugMask,
 *      DriverDbgLeaveFunc, DriverDbgLeaveFuncWithStatus
 */
#define DriverDbgEnterFunc(_module_) \
  do { \
      DriverDbgPrint(DBGLVLTRACE, \
                  (_module_), \
                  ( "[%s]: %s begin\n", (DRIVER_NAME), (__FUNCTION__) ) ); \
  } while(0)


/*! \def DriverDbgLeaveFunc(_module_)
 *  \brief Print name of function.
 * 
 *  This macro is useful with DriverDbgEnterFunc
 * 
 *  \param _module_ debug module
 *  \see DriverDbgEnterFunc, DriverDbgLeaveFuncWithStatus, DebugMask,
 *     DriverDbgPrint, DriverDbgSetDebugMask
 */
#define DriverDbgLeaveFunc(_module_) \
  do { \
      DriverDbgPrint(DBGLVLTRACE, \
                  (_module_), \
                  ( "[%s]: %s end\n\n", (DRIVER_NAME), (__FUNCTION__) ) ); \
  } while(0)


/*! \def DriverDbgLeaveFuncWithStatus(_module_, _status_)
 *  \brief Print name of function and leave status.
 *
 *  This macro is useful with DriverDbgEnterFunc
 *
 *  \param _module_ debug module
 *  \param _status_ leave status (NTSTATUS)
 *
 *  \see DriverDbgEnterFunc, DriverDbgLeaveFunc, DebugMask
 */
#define DriverDbgLeaveFuncWithStatus(_module_, _status_) \
  do { \
      DriverDbgPrint(DBGLVLTRACE, \
                     (_module_), \
                     ("[%s]: %s end (%s)\n\n", (DRIVER_NAME), \
                                                          (__FUNCTION__), \
                                                          OsrNTStatusToString( (_status_) )));\
  }while(0)


/*! \def DriverDbgPrintIrpName(_module_, _Irp_)
 *  \brief Print Irp name.
 *
 *  This macro prints Irp name, Major and Minor Function.
 *
 *  \param _module_ debug module
 *  \param _Irp_ pointer to IRP that we want inspect
 *
 *  \see DriverDbgPrintIrp, DebugMask, DriverDbgPrint
 */
#define DriverDbgPrintIrpName(_module_, _Irp_) \
  do { \
      PIO_STACK_LOCATION _pIoStack_ = (_Irp_)->Tail.Overlay.CurrentStackLocation; \
      DriverDbgPrint(DBGLVLTRACE, (_module_), ("[%s]: Major: %s, Minor: %s\n", \
                                               (DRIVER_NAME), \
                                               (IrpNames[(_pIoStack_)->MajorFunction]), \
											   ( ((_pIoStack_)->MajorFunction == IRP_MJ_PNP) ? PnpMinorFuncNames[(_pIoStack_)->MinorFunction] : 0))); \
  } while(0)


/*! \def DriverDbgPrintIrp(_module_, _Irp_)
 *  \brief Print Irp details.
 *
 *  This macro prints Irp details, like StackCount, CurrentLocation,
 *  address of DeviceObject and FileObject connected with this IRP.
 *
 *  \param _module_ debug module
 *  \param _Irp_ pointer to IRP that we want inspect
 *
 *  \see DriverDbgPrintIrpName
 */
#define DriverDbgPrintIrp(_module_, _Irp_) \
  do { \
      PIO_STACK_LOCATION _pIoStack_ = (_Irp_)->Tail.Overlay.CurrentStackLocation; \
      DriverDbgPrint(DBGLVLTRACE, (_module_), ("    StackCount=%d\n    CurrentLocation=%d\n", \
                                                (_Irp_)->StackCount, \
                                                (_Irp_)->CurrentLocation)); \
      DriverDbgPrint(DBGLVLTRACE, (_module_), ("    DeviceObject: 0x%X\n    FileObject: 0x%X\n", \
                                                (_pIoStack_)->DeviceObject, \
                                                (_pIoStack_)->FileObject)); \
  } while(0)


/*! \def DriverDbgError
 *  \brief Print Debug error message.
 *
 *  This macro print error message, when DebugMask is set to DBGLVLERROR, in all modules.
 *
 *  \param _format_ message format eg. ("some data %d\n", data)
 *  \see DebugMask, DriverDbgSetDebugMask
 */
#define DriverDbgError(_format_) \
  do { \
      DriverDbgPrint(DBGLVLERROR, \
                     DBGALLMODULES, \
                    ("[%s] file: %s, line: %d error -> ", (DRIVER_NAME), (__FILE__), (__LINE__)));\
      DriverDbgPrint(DBGLVLERROR, DBGALLMODULES, _format_); \
  }while(0)


/*! \def DriverDbgWarn(_format_)
 *  \brief Print Debug warining message.
 *
 * This macro print warning message, when DebugMask is set to DBGLVLWARN, in all modules.
 *
 * \param _format_ message format eg. ("some data %d\n", data)
 * \see DebugMask, DriverDbgSetDebugMask
 */
#define DriverDbgWarn(_format_) \
  do { \
      DriverDbgPrint(DBGLVLWARN, \
                     DBGALLMODULES, \
                     ("[%s] file: %s, line: %d warning -> ", (DRIVER_NAME), (__FILE__), (__LINE__))); \
      DriverDbgPrint(DBGLVLERROR, DBGALLMODULES, (_format_)); \
  }while(0)


/*! \def DriverDbgInfo(_format_)
 *  \brief Print Debug info message.
 *
 * This macro print info message, when DebugMask is set to DBGLVLINFO, in all modules.
 *
 * /param _format_ message format eg. ("some data %d\n", data)
 * /see DebugMask, DriverDbgSetDebugMask
 */
#define DriverDbgInfo(_format_) \
  do { \
      DriverDbgPrint(DBGLVLINFO, \
                     DBGALLMODULES, \
                     ("[%s] ", (DRIVER_NAME))); \
      DriverDbgPrint(DBGLVLINFO, \
                     DBGALLMODULES, \
                     (_format_)); \
  }while(0)


/*! \def DriverDbgTrace(_module_, _format_)
 *  \brief Print Debug trace message.
 *
 *  This macro print trace message, when DebugMask is set to DBGLVLTRACE, in all modules.
 *
 *  \param _module_ debug module
 *  \param _format_ message format eg. ("some data %d\n", data)
 *  \see DebugMask, DriverDbgSetDebugMask
 */
#define DriverDbgTrace(_module_, _format_) \
  do { \
      DriverDbgPrint(DBGLVLTRACE, (_module_), ("[%s]: ", (DRIVER_NAME))); \
      DriverDbgPrint(DBGLVLTRACE, (_module_), (_format_)); \
  }while(0)


/*! \def DriverInfo()
 *  \brief Print Driver information.
 *
 * This macro print driver specyfic information like driver name,
 * compliation date and compilation time.
 *
 */
#define DriverInfo()\
  do { \
    DbgPrint("\n[%s], date: %s, time: %s\n", (DRIVER_NAME), (__DATE__), (__TIME__)); \
  }while(0)


/*! \def DriverDbg(_format_)
 *  \brief Print debug message.
 *
 * This macro print driver name and debug message.
 *
 * \param _format_ format of message eg. ("something %d\n", data)
 *
 */
#define DriverDbg( _format_ ) \
  do { \
      DbgPrint("[%s]: ", (DRIVER_NAME)); \
      DbgPrint _format_; \
  }while(0)

#else // !DRIVER_DEBUG

#define DriverDbgPrint(_level_, _module_, _format_)
#define DriverDbgEnterFunc(_module_)
#define DriverDbgLeaveFunc(_module_)
#define DriverDbgPrintIrpName(_module_, _Irp_)
#define DriverDbgPrintIrp(_module_, _Irp_)
#define DriverDbgError(_format_)
#define DriverDbgWarn(_format_)
#define DriverDbgInfo(_format_)
#define DriverDbgTrace(_module_, _format_)
#define DriverInfo()
#define DriverDbgLeaveFuncWithStatus(_module_, _status_)
#define DriverDbg( _format_ )

#endif // DRIVER_DEBUG

#endif // __DEBUG_H__
